#!/bin/sh
# File: model2d.sh
# Script file to model shot gathers in (vel${m} model)
# This script will mute the data in offset domain and convert back to receiver domain
# shot${m}.H is the out data containing the info from all shots:
# axes are: 1=time, 2=rec_x, 3=sou_x

###############################
### How to use this script in makefile???:
# outputfile snapoutfile: vel${m}.H sou.HH Par/shot0.vel$(m).P ./Bin/model2d_program.x
#       sh model2d.sh '$(m)' 'outputfile' 'modelprogram' 'velfile' 'soufile' 'parfile' '$(abstop)' 'jsx=4' 'snapoutfile' 'tmpdir=.'
###############################
## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 

echo "calling $0 script"
echo "reading parameters..."

m=$1         #suffix names for showong identity of processing
echo m=$m

OUT=$2       #output file
if [ -z $OUT ];then echo 'OUT=$2 does not exist';exit;fi
echo OUT=$OUT 

MODELPROG=$3 #modeling program
if [ -z $MODELPROG ];then echo 'MODELPROG=$3 does not exist';exit;fi
echo MODELPROG=$MODELPROG

VELFILE=$4   #velocity file
if [ -z $VELFILE ];then echo 'VELFILE=$4 does not exist';exit;fi
echo VELFILE=$VELFILE

SOUFILE=$5   #source wavelet file
if [ -z $SOUFILE ];then echo 'SOUFILE=$5 does not exist';exit;fi
echo SOUFILE=$SOUFILE

PARFILE=$6   #parameter file for modeling program
if [ -z $PARFILE ];then echo 'PARFILE=$6 does not exist';exit;fi
echo PARFILE=$PARFILE
. ${PARFILE} # set necessary parameters, i.e. shot locations

abstop=$7    #whether or not to apply absorbing top boundary; if this does not exist, use abstop in PARFILE
echo abstop=$abstop

jsx=$8       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=4;fi
echo jsx=$jsx

SNAPOUT=$9  # wavfield snapshot file from the first shot
if [ -z $SNAPOUT ];then SNAPOUT=snap${m}.H;fi
echo SNAPOUT=$SNAPOUT

TEMPDIR=${10}   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

echo "reading parameters complete..."

###############################
#jsx=5  # increment between shots along x-axis  (the number of shot record to be muted in the same time)
#tdelay=8  # delayed time in seconds
###############################
#VELFILE=vel${m}.H
#OUT=shot${m}.H
#PARFILE=./Par/shot0.vel${m}.P
#MODELPROG=./Bin/model2d_program.x
#SOUFILE=sou.HH
#TEMPDIR=./temp   # temp file to store temporary data
# set the necessary parameters
#. ${PARFILE}


#<$< ${B}/model2d_program.x par=${P}/shot0.vel1.P abstop=0 source=sou.HH >$@ snap=shot0.wvfld1.mult.H

echo "nsx,dsx,osx,jsx" $nsx $dsx $osx $jsx

TEXT0=tmp0shot${m}.txt
TEXT1=tmp1shot${m}.txt
TEXT2=tmp2shot${m}.txt
TEXT3=tmp3shot${m}.txt
TEXT4=tmp4shot${m}.txt
TEXT5=tmp5shot${m}.txt
TEXT6=tmp6shot${m}.txt

OUT1=shot${m}out1.H
OUT2=shot${m}out2.H
OUT3=shot${m}out3.H
OUT4=shot${m}out4.H
OUT5=shot${m}out5.H
OUT6=shot${m}out6.H

Rm ${OUT} ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}


echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is modeling vel${m} shot record for ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"

for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do muting for ${jsx} shots at the same time

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run muting program within each combination of ${jsx} shots
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    sx=`echo $osx+$(($isx+$k-1))*$dsx | bc`
	    echo "****** modeling shot$((${isx}+${k})).... ******"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx $sx

	    # In the middle of each iteration we need to run the rtm program as background.
	    if [ $((${isx}+${k})) -eq 1 ]
	    then # The first shot might give us snapshot file
		<${VELFILE} ${MODELPROG} par=${PARFILE} nsx=1 osx=$sx abstop=${abstop} source=${SOUFILE} snap=${SNAPOUT} | Transp plane=13 | Transp plane=34 >${TEMPDIR}/shot${m}shot$((${isx}+${k})).H &
	    else
		<${VELFILE} ${MODELPROG} par=${PARFILE} nsx=1 osx=$sx get_snap=0 abstop=${abstop} source=${SOUFILE} | Transp plane=13 | Transp plane=34 >${TEMPDIR}/shot${m}shot$((${isx}+${k})).H &
	    fi

	    #save the list of files to be merged
	    if [ $((${isx}+${k})) -le 20 ]
	    then
		echo "${TEMPDIR}/shot${m}shot$((${isx}+${k})).H" >> ${TEXT1}
	    elif [ $((${isx}+${k})) -le 40 ]
	    then
		echo "${TEMPDIR}/shot${m}shot$((${isx}+${k})).H" >> ${TEXT2}
	    elif [ $((${isx}+${k})) -le 60 ]
	    then
		echo "${TEMPDIR}/shot${m}shot$((${isx}+${k})).H" >> ${TEXT3}
	    elif [ $((${isx}+${k})) -le 80 ]
	    then
		echo "${TEMPDIR}/shot${m}shot$((${isx}+${k})).H" >> ${TEXT4}
	    elif [ $((${isx}+${k})) -le 100 ]
	    then
		echo "${TEMPDIR}/shot${m}shot$((${isx}+${k})).H" >> ${TEXT5}
	    else
		echo "${TEMPDIR}/shot${m}shot$((${isx}+${k})).H" >> ${TEXT6}
	    fi
	fi
    done

    # wait untill all background processes to finish
    wait

done # end for loop


# loop end
if [ -f ${TEXT1} ]; then
    shotlist1=`cat ${TEXT1}`
    echo shotlist1=$shotlist1
    Merge axis=3 space=n $shotlist1 > ${OUT1} &
    echo ${OUT1} >> ${TEXT0}
fi
if [ -f ${TEXT2} ]; then
    shotlist2=`cat ${TEXT2}`
    echo shotlist2=$shotlist2
    Merge axis=3 space=n $shotlist2 > ${OUT2} &
    echo ${OUT2} >> ${TEXT0}
fi
if [ -f ${TEXT3} ]; then
    shotlist3=`cat ${TEXT3}`
    echo shotlist3=$shotlist3
    Merge axis=3 space=n $shotlist3 > ${OUT3} &
    echo ${OUT3} >> ${TEXT0}
fi
if [ -f ${TEXT4} ]; then
    shotlist4=`cat ${TEXT4}`
    echo shotlist4=$shotlist4
    Merge axis=3 space=n $shotlist4 > ${OUT4} &
    echo ${OUT4} >> ${TEXT0}
fi
if [ -f ${TEXT5} ]; then
    shotlist5=`cat ${TEXT5}`
    echo shotlist5=$shotlist5
    Merge axis=3 space=n $shotlist5 > ${OUT5} &
    echo ${OUT5} >> ${TEXT0}
fi
if [ -f ${TEXT6} ]; then
    shotlist6=`cat ${TEXT6}`
    echo shotlist6=$shotlist6
    Merge axis=3 space=n $shotlist6 > ${OUT6} &
    echo ${OUT6} >> ${TEXT0}
fi

# wait untill all background processes to finish
wait

shotlist0=`cat ${TEXT0}`
echo shotlist0=$shotlist0

Merge axis=3 space=n $shotlist0 >shot${m}tempyyy.H
<shot${m}tempyyy.H Transp plane=34 | Transp plane=13 > ${OUT}

# Remove files
Rm shot${m}tempyyy.H
Rm $shotlist0 $shotlist1 $shotlist2 $shotlist3 $shotlist4 $shotlist5 $shotlist6
rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}

exit
