#!/bin/sh
# File: mute2d.sh
# Script file to mute shot gathers in (vel4 model)
# This script will mute the data in offset domain and convert back to receiver domain
# mute1.H is the input data containing the info from all shots:
# axes are: 1=time, 2=rec_x, 3=sou_x


###############################
### How to use this script in makefile???:
# outputfile: in.H out.H Par/mute$(m).P
#       sh mute2d.sh '$(m)' 'inputfile' 'outputfile' 'parfile' 'jsx=8' 'tmpdir=.'
###############################
## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 
# input/output file must be n1=time, n2=receiver locations, n3=source locations

echo "calling $0 script"
echo "reading parameters..."

m=$1         #suffix names for showong identity of processing
echo m=$m

IN=$2        #input file
if [ -z $IN ];then echo 'IN=$2 does not exist';exit;fi
echo IN=$IN

OUT=$3       #output file
if [ -z $OUT ];then echo 'OUT=$3 does not exist';exit;fi
echo OUT=$OUT

PARFILE=$4   #parameter file for modeling program
if [ -z $PARFILE ];then echo 'PARFILE=$4 does not exist';exit;fi
echo PARFILE=$PARFILE

jsx=$5       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=8;fi
echo jsx=$jsx

TEMPDIR=$6   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

echo "reading parameters complete..."

#m=4 #name of model (vel${m})
###############################
#jsx=8  # increment between shots along x-axis  (the number of shot record to be muted in the same time)
###############################
#FILE=mute${m}.H
#PARFILE=./Par/mute${m}.P

TEXT=tmpmute${m}.txt

echo "Muting data in vel${m} model"
nsx=`Get < $IN parform=n n3`
dsx=`Get < $IN parform=n d3`
osx=`Get < $IN parform=n o3`
ogx=`Get < $IN parform=n o2`
dgx=`Get < $IN parform=n d2`
ogx=`Get < $IN parform=n o2`
echo "nsx,dsx,osx" $nsx $dsx $osx
echo "ngx,dgx,ogx" $ngx $dgx $ogx
echo "jsx" $jsx

Rm ${TEMPDIR/}mute${m}shot*.H
rm -f ${TEXT}

echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is muting data from ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"

for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do muting for ${jsx} shots at the same time

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run muting program within each combination of ${jsx} shots
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
            # window the data to get the data from shot${isx}
	    <$IN Window n3=1 f3=$((${isx}+${k}-1)) > ${TEMPDIR}/mute${m}shot$((${isx}+${k})).H

	    #### SORTING DATA from a single shot gather into related cmp gather
	    osx=`Get < ${TEMPDIR}/mute${m}shot$((${isx}+${k})).H parform=n o3`
	    ooffset=`echo $ogx-$osx | bc`
	    echo o2=$ooffset >> ${TEMPDIR}/mute${m}shot$((${isx}+${k})).H # convert receiver axis into offset axis
	    ####

	    echo "****** muting shot$((${isx}+${k})).... ******"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`

	    <${TEMPDIR}/mute${m}shot$((${isx}+${k})).H Mute par=$PARFILE > ${TEMPDIR}/mute${m}shot$((${isx}+${k})).muted.H &
	    echo "${TEMPDIR}/mute${m}shot$((${isx}+${k})).muted.H" >> ${TEXT}
	fi
    done

    # wait untill all background processes to finish
    wait

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/mute${m}shot$((${isx}+$k)).H &
            # convert back from offset domain to receiver domain
	    echo o2=$ogx >> ${TEMPDIR}/mute${m}shot$((${isx}+${k})).muted.H
	fi
    done

done # end for loop

mutelist=`cat ${TEXT}`
echo mutelist=$mutelist

Rm $OUT
Merge axis=3 space=n $mutelist > $OUT
Rm $mutelist
rm -f ${TEXT}

exit