#!/bin/sh
# File: rtm2d.sh
# 1st argument will be the value of macro ${m}, which is the name of velocity model vel${m}.mig.H
# Script file to parallelly compute RTM image (computing every ${jsx} shots along x-axis at the same time) 
# Input file: "rtm${m}in.H" (interpolated shot gathers)
# axes: 1=rec_z, 2=rec_x, 3=time, 4=sou_x, 5=sou_z
# Output file: "rtm${m}out.H"
# axes: 1=z, 2=x, 3=Undefined

###############################
### How to use this script in makefile???:
# out.H: in.H vel${m}.mig.H sou.HH Par/rtm0.vel$(m).P ./Bin/crtm2d_program.x
#       sh rtm2d.sh '$(m)' 'in.H' 'out.H' 'rtmprogram' 'vel.mig.H' 'soufile' 'parfile' 'jsx=4' 'tmpdir=.' 'sousnapfile.out=sousnap${m}.H' 'recsnapfile.out=recsnap${m}.H' 'reflsnapfile.out=reflsnap${m}.H'
###############################

## use double quatation marks "$NAME" to view the variable's contents
## single quatation marks '$NAME' does not show the contents of NAME 

echo "calling $0 script"
echo "reading parameters..."


m=$1         #suffix names for showong identity of processing
echo m=$m

IN=$2       #input file /gz,gx,t,sx,sz/ with interpreted gx
if [ -z $IN ];then echo 'IN=$2 does not exist';exit;fi
echo IN=$IN

OUT=$3       #output file /z,x/
if [ -z $OUT ];then echo 'OUT=$3 does not exist';exit;fi
echo OUT=$OUT 

RTMPROG=$4 #rtm program
if [ -z $RTMPROG ];then echo 'RTMPROG=$4 does not exist';exit;fi
echo RTMPROG=$RTMPROG

VELFILE=$5   #velocity file
if [ -z $VELFILE ];then echo 'VELFILE=$5 does not exist';exit;fi
echo VELFILE=$VELFILE

SOUFILE=$6   #source wavelet file
if [ -z $SOUFILE ];then echo 'SOUFILE=$6 does not exist';exit;fi
echo SOUFILE=$SOUFILE

PARFILE=$7   #parameter file for RTM program
if [ -z $PARFILE ];then echo 'PARFILE=$7 does not exist';exit;fi
echo PARFILE=$PARFILE
#. ${PARFILE} # set necessary parameters, i.e. shot locations

jsx=$8       #increment between shots along x-axis  (the number of shot record to be muted in the same time)
if [ -z $jsx ];then jsx=4;fi
echo jsx=$jsx

TEMPDIR=${9}   # temp file to store temporary data
if [ -z $TEMPDIR ];then TEMPDIR=.;fi
echo TEMPDIR=$TEMPDIR

SOUSNAPOUT=${10}  # source wavefield snapshot file
if [ -z $SOUSNAPOUT ];then SOUSNAPOUT=sousnap${m}.H;fi
echo SOUSNAPOUT=$SOUSNAPOUT

RECSNAPOUT=${11}  # receiver wavefield snapshot file
if [ -z $RECSNAPOUT ];then RECSNAPOUT=recsnap${m}.H;fi
echo RECSNAPOUT=$RECSNAPOUT

SNAPOUT=${12}  # correlated image snapshot file
if [ -z $SNAPOUT ];then SNAPOUT=reflsnap${m}.H;fi
echo SNAPOUT=$SNAPOUT

echo "reading parameters complete..."


#m=$1 #name of model (vel1)
###############################
#jsx=4  # increment between shots along x-axis  (the intershots will be RTMed at the same time)
###############################

#IN=rtm${m}in.H
#OUT=rtm${m}out.H
#SOUSNAPOUT=sousnap${m}.H
#RECSNAPOUT=recsnap${m}.H
#SNAPOUT=reflsnap${m}.H
#PARFILE=./Par/rtm0.vel${m}.P
#RTMPROG=./Bin/crtm2d_program.x
#TEMPDIR=./temp
#VELFILE=vel${m}.mig.H
#SOUFILE=sou.HH

TEXT0=tmp0rtm${m}.txt
TEXT1=tmp1rtm${m}.txt
TEXT2=tmp2rtm${m}.txt
TEXT3=tmp3rtm${m}.txt
TEXT4=tmp4rtm${m}.txt
TEXT5=tmp5rtm${m}.txt
TEXT6=tmp6rtm${m}.txt

OUT1=rtm${m}out1.H
OUT2=rtm${m}out2.H
OUT3=rtm${m}out3.H
OUT4=rtm${m}out4.H
OUT5=rtm${m}out5.H
OUT6=rtm${m}out6.H

Rm ${TEMPDIR}/rtm${m}imageshot*.H
Rm ${TEMPDIR}/rtm${m}shotrecord*.H
Rm ${OUT} ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}

echo "Start program to compute RTM image in vel1 model..."
echo "Reading shot info..."
nsx=`Get < $IN parform=n n4`
dsx=`Get < $IN parform=n d4`
osx=`Get < $IN parform=n o4`
echo "nsx,dsx,osx,jsx" $nsx $dsx $osx $jsx

echo "5555555555555555555555555555555555555555555555555555555555555555555555"
echo "The script is computing RTM images from ${jsx} shots at the same time."
echo "5555555555555555555555555555555555555555555555555555555555555555555555"
for((isx=1; $isx<=$nsx; isx=$isx+$jsx))
do # each iteration will do rtm for ${jsx} shots at the same time

    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]
	then
	    echo "************* Windowing and Transposing SHOT$((${isx}+$k)) *****************"
	    echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`
	    # This set of command will consume 2 cpu's at the same time; one might guess tha the simutaneous loops should not be greater than half of #cpu
	    # BUT computing more than half of #cpu simutaneouly is significantly more effective than limiting ourselves to just 4 simutaneous loops.
	    # SO, what I did here is the best way. No need to window and transpose the whole shots first with 4-simutaneous process before entering RTM with ${jsx}-simutaneous process
	    # Note that calling only-Window commands simutaneously will share the whole processes in a single CPU. BUT calling only-Transp commands simutaneously will separate every single process to a different CPU. SO, if we make Window-Transp dependent to each other, it will behave like only-Transp. That's why I use .... Window | Transp .....
	    <${IN} Window n4=1 f4=$((${isx}+${k}-1)) | Transp plane=12 reshape=2,3,5 > ${TEMPDIR}/rtm${m}shotrecord$((${isx}+${k})).H &
	fi
    done
    wait

    #############
    #### do RTM imaging ${jsx} shots in the same time
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do # run rtm program within each combination of ${jsx} shots

	if [ $((${isx}+${k})) -le ${nsx} ]
	then

	    #echo "************* Windowing and Transposing SHOT$((${isx}+$k)) *****************"
	    #echo "isx,osx,sx:" $(($isx+$k)) $osx `echo $osx+$(($isx+$k-1))*$dsx | bc`
	    #<${IN} Window n4=1 f4=$((${isx}+${k}-1)) | Transp plane=12 reshape=2,3,5 > ${TEMPDIR}/rtm${m}shotrecord$((${isx}+${k})).H

	    echo "************* RTM Imaging SHOT$((${isx}+$k)) *****************"
	    if [ $((${isx}+${k})) -eq 1 ]
	    then # The first shot might give us snapshot file
		<${VELFILE} ${RTMPROG} par=${PARFILE} source=${SOUFILE} receiver=${TEMPDIR}/rtm${m}shotrecord$((${isx}+${k})).H >${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H snap=${SNAPOUT} sousnap=${SOUSNAPOUT} recsnap=${RECSNAPOUT} &
	    else
		<${VELFILE} ${RTMPROG} par=${PARFILE} get_snap=0 get_sousnap=0 get_recsnap=0 source=${SOUFILE} receiver=${TEMPDIR}/rtm${m}shotrecord$((${isx}+${k})).H > ${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H &
	    fi

	    if [ $((${isx}+${k})) -le 20 ]
	    then
		echo "${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT1}
	    elif [ $((${isx}+${k})) -le 40 ]
	    then
		echo "${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT2}
	    elif [ $((${isx}+${k})) -le 60 ]
	    then
		echo "${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT3}
	    elif [ $((${isx}+${k})) -le 80 ]
	    then
		echo "${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT4}
	    elif [ $((${isx}+${k})) -le 100 ]
	    then
		echo "${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT5}
	    else
		echo "${TEMPDIR}/rtm${m}imageshot$((${isx}+${k})).H" >> ${TEXT6}
	    fi
	fi
    done
    
    # wait untill all background processes to finish
    wait

    # remove temporary windowed- and transposed-inputdata
    for((k=0; $k<=$((${jsx}-1)); k=$k+1))
    do
	if [ $((${isx}+${k})) -le ${nsx} ]; then
	    Rm ${TEMPDIR}/rtm${m}shotrecord$((${isx}+$k)).H
	fi
    done

done

# loop end
if [ -f ${TEXT1} ]; then
    rtmlist1=`cat ${TEXT1}`
    echo rtmlist1=$rtmlist1
    Merge axis=3 space=n $rtmlist1 | Transp plane=23 | Stack axes=2 > ${OUT1} &
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT1}
    echo ${OUT1} >> ${TEXT0}
    runloop=1
fi
if [ -f ${TEXT2} ]; then
    rtmlist2=`cat ${TEXT2}`
    echo rtmlist2=$rtmlist2
    Merge axis=3 space=n $rtmlist2 | Transp plane=23 | Stack axes=2 > ${OUT2} &
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT2}
    echo ${OUT2} >> ${TEXT0}
    runloop=2
fi
if [ -f ${TEXT3} ]; then
    rtmlist3=`cat ${TEXT3}`
    echo rtmlist3=$rtmlist3
    Merge axis=3 space=n $rtmlist3 | Transp plane=23 | Stack axes=2 > ${OUT3} &
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT3}
    echo ${OUT3} >> ${TEXT0}
    runloop=3
fi
if [ -f ${TEXT4} ]; then
    rtmlist4=`cat ${TEXT4}`
    echo rtmlist4=$rtmlist4
    Merge axis=3 space=n $rtmlist4 | Transp plane=23 | Stack axes=2 > ${OUT4} &
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT4}
    echo ${OUT4} >> ${TEXT0}
    runloop=4
fi
if [ -f ${TEXT5} ]; then
    rtmlist5=`cat ${TEXT5}`
    echo rtmlist5=$rtmlist5
    Merge axis=3 space=n $rtmlist5 | Transp plane=23 | Stack axes=2 > ${OUT5} &
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT5}
    echo ${OUT5} >> ${TEXT0}
    runloop=5
fi
if [ -f ${TEXT6} ]; then
    rtmlist6=`cat ${TEXT6}`
    echo rtmlist6=$rtmlist6
    Merge axis=3 space=n $rtmlist6 | Transp plane=23 | Stack axes=2 > ${OUT6} &
    echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT6}
    echo ${OUT6} >> ${TEXT0}
    runloop=6
fi

# wait untill all background processes to finish
wait

rtmlist0=`cat ${TEXT0}`
echo rtmlist0=$rtmlist0

# Merge all RTM images together
Merge axis=3 space=n $rtmlist0 |Transp plane=23|Stack axes=2 > ${OUT}
echo o3=${osx} d3=${dsx} label3="sou_x" >> ${OUT}

## Remove temporary files
Rm $rtmlist0 $rtmlist1 $rtmlist2 $rtmlist3 $rtmlist4 $rtmlist5 $rtmlist6
Rm ${OUT1} ${OUT2} ${OUT3} ${OUT4} ${OUT5} ${OUT6}
rm -f ${TEXT0} ${TEXT1} ${TEXT2} ${TEXT3} ${TEXT4} ${TEXT5} ${TEXT6}

exit