!	   		        Padntape_program
!
!           Pad a dataset with traces of tapered nearest value
!
!		Author: Kittinat Taweesintananon
!
! Example:
!
! <in.H n1= n2= n3= extend=0 Padntape_program.x >out.H  beg[] end[]
!
!	Necessary input files:
!
!		vel.H is the Header for the velocity model, stdin: dimension /z x/
!
! **Necessary parameters****************************************************
!n1out,nxout - int
!          [nx]   Add nxout-nx-begx to the end of the cube dimension
!                   (such that the total dimension along the x axis will be nxout).
!                  [0] will pad to the next power of 2
!
!begx - int
!           [0]    Add begx to the begining of axis x (>=0)
!
!verb - int
!           [0]   Whether (1) or not (0) to be verbose
!
!extend - int
!           [0]   Whether to extend with the tapered zone (2), the nearest point (1) or zeros (0)
!
!taper - real
!           [97]  percentage of the element at the end of tapered zone with respect to the nearest point (true input)
!
!esize - int
!           [4]   real(4) or complex([8])
!
!!==============================================================================


program Padntape_program

  use sep
  implicit none

  logical           :: verb
  integer           :: n1,n2,n3, i,j,k, n1out,n2out,n3out, beg1,beg2,beg3, end1,end2,end3, extend,esize
  real              :: o1,o2,o3, d1,d2,d3, delp,taper
  character(len=128)  :: label1,label2,label3

  real, dimension(:,:,:), allocatable    :: in1,out1    !/n1,n2,n3/
  complex, dimension(:,:,:), allocatable :: in2,out2    !/n1,n2,n3/

  call sep_init()
  !! I/O stuff
  ! get parameters from the input history file
    call from_history("n1",n1)
    call from_history("d1",d1)
    call from_history("o1",o1,0.)
    call from_history('label1',label1,'z')
    ! if there exists the 2nd axis
    call from_history("n2",n2,1)
    call from_history("d2",d2,10.)
    call from_history("o2",o2,0.)
    call from_history('label2',label2,'x')
    ! if there exists the 3rd axis
    call from_history("n3",n3,1)
    call from_history("d3",d3,10.)
    call from_history("o3",o3,0.)
    call from_history('label3',label3,'y')
    write(0,*) "Reading axes complete...."

    !! read parameters

!begx - int
!           [0]    Add begx to the begining of axis x
!
!verb - int
!           [0]   Whether (1) or not (0) to be verbose
!
!extend - int
!           [0]   Whether to extend with the nearest point (1) or zeros (0)
!
!esize - int
!           [4]   real[4] or complex[8]

    call from_param("n1out",n1out,n1) !
    call from_param("n2out",n2out,n2) !
    call from_param("n3out",n3out,n3) !
    call from_param("beg1",beg1,0) !
    call from_param("beg2",beg2,0) !
    call from_param("beg3",beg3,0) !
    call from_param("taper",taper,97.)
    call from_param("verb",verb,.false.)
    call from_param("extend",extend,0)
    call from_param("esize",esize,4)

    ! check if the parameters are correct
    if(extend.ne.0 .and. extend.ne.1 .and. extend.ne.2) call seperr("'extend' must be 0 or 1 or 2")
    if(esize.ne.4 .and. esize.ne.8) call seperr("'esize' must be 4 or 8")

    end1 = n1out-beg1-n1
    end2 = n2out-beg2-n2
    end3 = n3out-beg3-n3

    !! Illustrate input parameters
    write(0,*) '**************************************************'
    write(0,*) '*************** Input parameters.. ***************'
    write(0,*) '**************************************************'
    write(0,*) 'n1,d1,o1,label1', n1, d1, o1, label1
    write(0,*) 'n2,d2,o2,label2', n2, d2, o2, label2
    write(0,*) 'n3,d3,o3,label3', n3, d3, o3, label3
    write(0,*) 'beg1,beg2,beg3', beg1,beg2,beg3
    write(0,*) 'end1,end2,end3', end1,end2,end3
    write(0,*) "extend,esize,verb", extend,esize,verb
    write(0,*) '**************************************************'
    write(0,*) 'memmory(kB) ~ n1*n2*n3*4', (n1*n2*n3*4)/1000
    write(0,*) '**************************************************'

    !! write axes parameters to the output history file
    call to_history("n1",n1out)
    call to_history("d1",d1)
    call to_history("o1",o1)
    call to_history("label1",label1)
    call to_history("n2",n2out)
    call to_history("d2",d2)
    call to_history("o2",o2)
    call to_history("label2",label2)
    call to_history("n3",n3out)
    call to_history("d3",d3)
    call to_history("o3",o3)
    call to_history("label3",label3)
    write(0,*) "Writing axes complete...."

    !! Illustrate output parameters
    write(0,*) '**************************************************'
    write(0,*) '*************** Output parameters.. ***************'
    write(0,*) '**************************************************'
    write(0,*) 'n1out,d1,o1,label1', n1out, d1, o1, label1
    write(0,*) 'n2out,d2,o2,label2', n2out, d2, o2, label2
    write(0,*) 'n3out,d3,o3,label3', n3out, d3, o3, label3
    write(0,*) '**************************************************'

  call sep_close()

  ! Allocate parameters
  if(esize.eq.4)then
     allocate(in1(n1,n2,n3),out1(n1out,n2out,n3out))
     in1=0.;out1=0.;
  elseif(esize.eq.8)then
     allocate(in2(n1,n2,n3),out2(n1out,n2out,n3out))
     in2=(0.,0.);out2=(0.,0.);
  end if
  write(0,*) "allocation completed"

  !! our work begins here
  !! read data
  if(esize.eq.4)then
     call sep_read(in1)
  elseif(esize.eq.8)then
     call sep_read(in2)
  end if
  write(0,*) "data have been read"

  !!!==== Work begins here ====



if(esize.eq.4)then   ! real data

  do k=1,n3
     do j=1,n2
        do i=1,n1
           out1(beg1+i,beg2+j,beg3+k) = in1(i,j,k)
        end do
     end do
  end do

  if(extend.eq.1 .or. extend.eq.2)then
     do k=1,n3
        do j=1,n2
           do i=1,beg1+1
              out1(i,j+beg2,k+beg3) = in1(1,j,k)  ! which equals out1(beg1+1,j,k)
           end do
           do i=n1+beg1,n1out
              out1(i,j+beg2,k+beg3) = in1(n1,j,k)  ! which equals out1(beg1+n1,j,k)            
           end do
        end do
     end do
     do k=1,n3
        do i=1,n1
           do j=1,beg2+1
              out1(i+beg1,j,k+beg3) = in1(i,1,k)  ! which equals out1(i,beg2+1,k)
           end do
           do j=n2+beg2,n2out
              out1(i+beg1,j,k+beg3) = in1(i,n2,k)  ! which equals out1(i,beg2+n2,k)            
           end do
        end do
     end do
     do j=1,n2
        do i=1,n1
           do k=1,beg3+1
              out1(beg1+i,beg2+j,k) = in1(i,j,1)
           end do
           do k=n3+beg3,n3out
              out1(beg1+i,beg2+j,k) = in1(i,j,n3)
           end do
        end do
     end do

     !! extend at corners !!!!
     do k=1,n3
        out1(1:beg1+1,1:beg2+1,beg3+k) = in1(1,1,k)
        out1(1:beg1+1,n2+beg2:n2out,beg3+k) = in1(1,n2,k)
        out1(n1+beg1:n1out,1:beg2+1,beg3+k) = in1(n1,1,k)
        out1(n1+beg1:n1out,n2+beg2:n2out,beg3+k) = in1(n1,n2,k)
     end do
     do j=1,n2
        out1(1:beg1+1,beg2+j,1:beg3+1) = in1(1,j,1)
        out1(1:beg1+1,beg2+j,n3+beg3:n3out) = in1(1,j,n3)
        out1(n1+beg1:n1out,beg2+j,1:beg3+1) = in1(n1,j,1)
        out1(n1+beg1:n1out,beg2+j,n3+beg3:n3out) = in1(n1,j,n3)
     end do
     do i=1,n1
        out1(beg1+i,1:beg2+1,1:beg3+1) = in1(i,1,1)
        out1(beg1+i,1:beg2+1,n3+beg3:n3out) = in1(i,1,n3)
        out1(beg1+i,n2+beg2:n2out,1:beg3+1) = in1(i,n2,1)
        out1(beg1+i,n2+beg2:n2out,n3+beg3:n3out) = in1(i,n2,n3)
     end do


     if(extend.eq.2)then
        !! taper axis1
        if(end1.ne.0)then
           delp=-log(taper*0.01)/real(end1)**2  ! -ln(%)=-0.0513/xmax**2
           do k=1,n3out       !beg3+1,beg3+n3
              do j=1,n2out !beg2+1,beg2+n2
                 do i=n1+beg1,n1out   !n1out=n1+beg1+end1
                    out1(i,j,k) = out1(i,j,k)*exp(-delp*float(i - (n1+beg1))**2)
                 end do
              end do
           end do
        end if
        !
        if(beg1.ne.0)then
           delp=-log(taper*0.01)/real(beg1)**2  ! -ln(%)=-0.0513/xmax**2
           do k=1,n3out!beg3+1,beg3+n3
              do j=1,n2out!beg2+1,beg2+n2
                 do i=1,beg1+1
                    out1(i,j,k) = out1(i,j,k)*exp(-delp*float(i - (1+beg1))**2)
                 end do
              end do
           end do
        end if
        !! taper axis2
        if(end2.ne.0)then
           delp=-log(taper*0.01)/real(end2)**2  ! -ln(%)=-0.0513/xmax**2
           do k=1,n3out!beg3+1,beg3+n3
              do i=1,n1out!beg1+1,beg1+n1
                 do j=n2+beg2,n2out   !n2out=n2+beg2+end2
                    out1(i,j,k) = out1(i,j,k)*exp(-delp*float(j - (n2+beg2))**2)
                 end do
              end do
           end do
        end if
        !
        if(beg2.ne.0)then
           delp=-log(taper*0.01)/real(beg2)**2  ! -ln(%)=-0.0513/xmax**2
           do k=1,n3out!beg3+1,beg3+n3
              do i=1,n1out!beg1+1,beg1+n1
                 do j=1,beg2+1   !n2out=n2+beg2+end2
                    out1(i,j,k) = out1(i,j,k)*exp(-delp*float(j - (1+beg2))**2)
                 end do
              end do
           end do
        end if
        !! taper axis3
        if(end3.ne.0)then
           delp=-log(taper*0.01)/real(end3)**2  ! -ln(%)=-0.0513/xmax**2
           do j=1,n2out!beg2+1,beg2+n2
              do i=1,n1out!beg1+1,beg1+n1
                 do k=n3+beg3,n3out   !n1out=n1+beg1+end1
                    out1(i,j,k) = out1(i,j,k)*exp(-delp*float(k - (n3+beg3))**2)
                 end do
              end do
           end do
        end if
        !
        if(beg3.ne.0)then
           delp=-log(taper*0.01)/real(beg3)**2  ! -ln(%)=-0.0513/xmax**2
           do j=1,n2out!beg2+1,beg2+n2
              do i=1,n1out!beg1+1,beg1+n1
                 do k=1,beg3+1   !n1out=n1+beg1+end1
                    out1(i,j,k) = out1(i,j,k)*exp(-delp*float(k - (1+beg3))**2)
                 end do
              end do
           end do
        end if
     end if
  end if

elseif(esize.eq.8)then  ! complex data

  do k=1,n3
     do j=1,n2
        do i=1,n1
           out2(beg1+i,beg2+j,beg3+k) = in2(i,j,k)
        end do
     end do
  end do

  if(extend.eq.1 .or. extend.eq.2)then
     do k=1,n3
        do j=1,n2
           do i=1,beg1+1
              out2(i,j+beg2,k+beg3) = in2(1,j,k)  ! which equals out2(beg1+1,j,k)
           end do
           do i=n1+beg1,n1out
              out2(i,j+beg2,k+beg3) = in2(n1,j,k)  ! which equals out2(beg1+n1,j,k)            
           end do
        end do
     end do
     do k=1,n3
        do i=1,n1
           do j=1,beg2+1
              out2(i+beg1,j,k+beg3) = in2(i,1,k)  ! which equals out2(i,beg2+1,k)
           end do
           do j=n2+beg2,n2out
              out2(i+beg1,j,k+beg3) = in2(i,n2,k)  ! which equals out2(i,beg2+n2,k)            
           end do
        end do
     end do
     do j=1,n2
        do i=1,n1
           do k=1,beg3+1
              out2(beg1+i,beg2+j,k) = in2(i,j,1)
           end do
           do k=n3+beg3,n3out
              out2(beg1+i,beg2+j,k) = in2(i,j,n3)
           end do
        end do
     end do

     !! extend at corners !!!!
     do k=1,n3
        out2(1:beg1+1,1:beg2+1,beg3+k) = in2(1,1,k)
        out2(1:beg1+1,n2+beg2:n2out,beg3+k) = in2(1,n2,k)
        out2(n1+beg1:n1out,1:beg2+1,beg3+k) = in2(n1,1,k)
        out2(n1+beg1:n1out,n2+beg2:n2out,beg3+k) = in2(n1,n2,k)
     end do
     do j=1,n2
        out2(1:beg1+1,beg2+j,1:beg3+1) = in2(1,j,1)
        out2(1:beg1+1,beg2+j,n3+beg3:n3out) = in2(1,j,n3)
        out2(n1+beg1:n1out,beg2+j,1:beg3+1) = in2(n1,j,1)
        out2(n1+beg1:n1out,beg2+j,n3+beg3:n3out) = in2(n1,j,n3)
     end do
     do i=1,n1
        out2(beg1+i,1:beg2+1,1:beg3+1) = in2(i,1,1)
        out2(beg1+i,1:beg2+1,n3+beg3:n3out) = in2(i,1,n3)
        out2(beg1+i,n2+beg2:n2out,1:beg3+1) = in2(i,n2,1)
        out2(beg1+i,n2+beg2:n2out,n3+beg3:n3out) = in2(i,n2,n3)
     end do

     if(extend.eq.2)then
        !! taper axis1
        delp=-log(taper*0.01)/real(end1)**2  ! -ln(%)=-0.0513/xmax**2
        do k=beg3+1,beg3+n3
           do j=beg2+1,beg2+n2
              do i=n1+beg1,n1out   !n1out=n1+beg1+end1
                 out2(i,j,k) = out2(i,j,k)*exp(-delp*float(i - (n1+beg1))**2)
              end do
           end do
        end do
        !
        delp=-log(taper*0.01)/real(beg1)**2  ! -ln(%)=-0.0513/xmax**2
        do k=beg3+1,beg3+n3
           do j=beg2+1,beg2+n2
              do i=1,beg1+1
                 out2(i,j,k) = out2(i,j,k)*exp(-delp*float(i - (1+beg1))**2)
              end do
           end do
        end do
        !! taper axis2
        delp=-log(taper*0.01)/real(end2)**2  ! -ln(%)=-0.0513/xmax**2
        do k=beg3+1,beg3+n3
           do i=beg1+1,beg1+n1
              do j=n2+beg2,n2out   !n2out=n2+beg2+end2
                 out2(i,j,k) = out2(i,j,k)*exp(-delp*float(j - (n2+beg2))**2)
              end do
           end do
        end do
        !
        delp=-log(taper*0.01)/real(beg2)**2  ! -ln(%)=-0.0513/xmax**2
        do k=beg3+1,beg3+n3
           do i=beg1+1,beg1+n1
              do j=1,beg2+1   !n2out=n2+beg2+end2
                 out2(i,j,k) = out2(i,j,k)*exp(-delp*float(j - (1+beg2))**2)
              end do
           end do
        end do
        !! taper axis3
        delp=-log(taper*0.01)/real(end3)**2  ! -ln(%)=-0.0513/xmax**2
        do j=beg2+1,beg2+n2
           do i=beg1+1,beg1+n1
              do k=n3+beg3,n3out   !n1out=n1+beg1+end1
                 out2(i,j,k) = out2(i,j,k)*exp(-delp*float(k - (n3+beg3))**2)
              end do
           end do
        end do
        !
        delp=-log(taper*0.01)/real(beg3)**2  ! -ln(%)=-0.0513/xmax**2
        do j=beg2+1,beg2+n2
           do i=beg1+1,beg1+n1
              do k=1,beg3+1   !n1out=n1+beg1+end1
                 out2(i,j,k) = out2(i,j,k)*exp(-delp*float(k - (1+beg3))**2)
              end do
           end do
        end do
     end if
  end if

end if

  !!!==========================


  !! write data
  if(esize.eq.4)then
     call sep_write(out1)
  elseif(esize.eq.8)then
     call sep_write(out2)
  end if
  write(0,*) "data have been written"

  !! work ends here

  !deallocate parameters
  if(esize.eq.4)then
     deallocate(in1,out1)
  elseif(esize.eq.8)then
     deallocate(in2,out2)
  end if
  write(0,*) "Deallocation completed"

end program Padntape_program
