module fftshift_mod
  
  use adj_mod
  implicit none
  integer, private :: i

contains

  !!!!******************************************************

  subroutine fftshift1Dr_init()
  end subroutine fftshift1Dr_init

  function fftshift1Dr_lop(adj,add,x,y) result(stat)
    ! This function is endowed with the real-numbered arrays
    ! adj=.false. means y is the interpretable result of swaping the left and right halves of the original FFTW'ed x
    ! x=model is the complex array able to work with FFTW.
    ! y=data is the interpretable versions of FFTW'ed data: y shows both negative and positive values of fourier domain.

    integer :: stat
    logical,intent(in) :: adj,add
    real, dimension(:) :: x,y
    integer            :: n
    if(size(x).ne.size(y))then; write(0,*)"ERROR size of x and y must be equal";stop;end if
    n=size(x)
    !write(0,*) "n,n/2,size(x),size(y)", n,n/2,size(x),size(y)

    call adjnull(adj,add,x,y)
    !if(.not.add) then
    !   if(adj)then
    !      x=(0.,0.)
    !   else
    !      y=(0.,0.)
    !   end if
    !end if

    if(adj)then ! adj=.true. x+=f(y)
       x = x + cshift(y,shift=n/2)
    else !adj=.false. y+=f(x)
       y = y + cshift(x,shift=-n/2)
    end if
    !write(0,*) "Shifting complete........."
    stat=0

  end function fftshift1Dr_lop

  subroutine fftshift1Dr_close()
  end subroutine fftshift1Dr_close

  !!!!******************************************************

  function fftshift1Dc_lop(adj,add,x,y) result(stat)
    ! This function is the same as above but the arrays are complex numbers
    ! If adj=.true., x=x(y) which is an original FFT array in [0,2*fnyq) (so it can be used in FFTW).
    ! If adj=.false. means y=y(x) is the interpretable result in [-fnyq,fnyq) of swaping the left and right halves of the original FFTW'ed x
    ! x=model is the complex array able to work with FFTW.
    ! y=data is the interpretable versions of FFTW'ed data: y shows both negative and positive values of fourier domain.

    integer :: stat
    logical,intent(in) :: adj,add
    complex, dimension(:) :: x,y
    integer            :: n,nhalf
    if(size(x).ne.size(y))then
       write(0,*)"ERROR size of x and y must be equal"
       stop
    end if
    n=size(x)
    nhalf=(n+1)/2
    !write(0,*) "n,n/2", n,n/2

    !call adjnull(adj,add,x,y)
    if(.not.add) then
       if(adj)then
          x=(0.,0.)
       else
          y=(0.,0.)
       end if
    end if

    if(adj)then ! adj=.true. x+=f(y)
       x = x + cshift(y,shift=n/2)
    else !adj=.false. y+=f(x)
       y = y + cshift(x,shift=-n/2)
    end if

    !if(mod(n,2).eq.0)then ! n is even
    !   write(0,*) "size of array is even"
    !   if(2*nhalf .ne. n) call seperr("2*nhalf must equal n")
    !   if(adj)then ! adj=.true. x+=f(y)
    !      do i=1,nhalf
    !         x(i+nhalf) = x(i+nhalf) + y(i)
    !         x(i) = x(i) + y(i+nhalf)
    !      end do
    !   else !adj=.false. y+=f(x)
    !      do i=1,nhalf
    !         y(i) = y(i) + x(i+nhalf)
    !         y(i+nhalf) = y(i+nhalf) + x(i)
    !      end do
    !   end if
    !else ! n is odd
    !   write(0,*) "size of array is odd"
    !   if(2*nhalf-1 .ne. n) call seperr("2*nhalf-1 must equal n")
    !   if(adj)then ! adj=.true. x+=f(y)
    !      do i=1,nhalf-1
    !         x(i+nhalf) = x(i+nhalf) + y(i)
    !         x(i) = x(i) + y(i+nhalf-1)
    !      end do
    !      x(nhalf) = x(nhalf) + y(n)
    !   else !adj=.false. y+=f(x)
    !      do i=1,nhalf-1
    !         y(i) = y(i) + x(i+nhalf)
    !         y(i+nhalf-1) = y(i+nhalf-1) + x(i)
    !      end do
    !      y(n) = y(n) + x(nhalf)
    !   end if
    !end if
    !write(0,*) "Shifting complete........."
    stat=0

  end function fftshift1Dc_lop

end module
