module fkfilter3D_mod
  
  implicit none

  integer, private  :: pi=3.14159265

contains

  subroutine filter3D(data,mode,flag,alpha,phi)
    !! mode=0 -- do nothing
    !! mode=1 -- sharp cut
    !! mode=2 -- Tukey filter using alpha parameter and fan filter with Tukey window
    !         # /t,z,x/
    !         # flag=1 keep the data in (+f,+kz,+kx) and (-f,-kz,-kx): downright-going wavefields
    !         # flag=-1 keep the data in (+f,-kz,-kx) and (-f,+kz,+kx): upleft-going wavefields
    !         # flag=2 keep the data in (+f,-kz,+kx) and (-f,+kz,-kx): upright-going wavefields
    !         # flag=-2 keep the data in (+f,+kz,-kx) and (-f,-kz,+kx): downleft-going wavefields

    complex,dimension(:,:,:),intent(inout) :: data
    integer                              :: mode, flag, n1, n2, n3, nn1,nn2,nn3, i,j,k
    real,intent(inout),optional          :: alpha,phi  ! input is degree, output=radian

    if( .not.present(alpha) .and. mode.eq.2 ) then
       print *, "WARNING: alpha are not present when using mode2 in filter1Dfn()"
       alpha = 0.2   ! alpha=0.2 is recommended
    end if
    if( .not.present(phi) .and. mode.eq.2) then
       print *, "WARNING: phi is not present when using mode2 in filter2Dfn()"
       phi=5.  ! 5' is recommended
    end if
    phi = phi*pi/180.  ! convert from degree to radian

    !! size of input data
    n1=size(data,1)
    n2=size(data,2)
    n3=size(data,3)

    if(mode.eq.0) then ! do nothing
       data=data
    end if
    
    if(mode.eq.1) then ! step cut
       if(flag.eq.1) then
!         # flag=1 keep the data in (+f,+kz,+kx) and (-f,-kz,-kx): downright-going wavefields
          ! flag=1: (+,+,+)axes and (-,-,-)axes will be stored, otherwise will be removed.
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = 0. ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = 0. ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = 0. ! (+,-,+)

          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = 0. ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = 0. ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = data(1:n1/2,1:n2/2,1:n3/2) ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = 0. ! (-,-,+)

       elseif(flag.eq.-1)then
!         # flag=-1 keep the data in (+f,-kz,-kx) and (-f,+kz,+kx): upleft-going wavefields
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = 0. ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = 0. ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = data(n1/2+1:n1,1:n2/2,1:n3/2) ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = 0. ! (+,-,+)

          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = data(1:n1/2,n2/2+1:n2,n3/2+1:n3) ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = 0. ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = 0. ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = 0. ! (-,-,+)

       elseif(flag.eq.2)then
!         # flag=2 keep the data in (+f,-kz,+kx) and (-f,+kz,-kx): upright-going wavefields
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = 0. ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = 0. ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = 0. ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = data(n1/2+1:n1,1:n2/2,n3/2+1:n3) ! (+,-,+)

          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = 0. ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = data(1:n1/2,n2/2+1:n2,1:n3/2) ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = 0. ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = 0. ! (-,-,+)

       elseif(flag.eq.-2)then
!         # flag=-2 keep the data in (+f,+kz,-kx) and (-f,-kz,+kx): downleft-going wavefields
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = 0. ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = data(n1/2+1:n1,n2/2+1:n2,1:n3/2) ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = 0. ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = 0. ! (+,-,+)

          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = 0. ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = 0. ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = 0. ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = data(1:n1/2,1:n2/2,n3/2+1:n3) ! (-,-,+)
       end if
  
    end if

    if(mode.eq.2)then
       !!============================
       !! apply 2D cosine fan filter on the plane composed of the last two axes
       !!============================

       if(alpha .gt. 1.0) then
          print *, "ERROR: alpha must be less than 1 for Tukey window function!!"
          call exit()
       end if

       !!============================
       !! apply Tukey taper on the 1st axis
       !! positive 1st axis
       nn1=n1-int(n1/2)-1 ! number of positive elements on the 1st axis
       do i=0,nn1
          if( i .lt. int(0.5*alpha*float(nn1)) )then
             data(n1/2+1+i,:,:) = data(n1/2+1+i,:,:)*( 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn1)) - 1.0 )) ) ! (+,:,:)
          elseif( i.ge.int(0.5*alpha*float(nn1)) .and. i.le.(nn1-int(0.5*alpha*float(nn1))) )then
             data(n1/2+1+i,:,:) = data(n1/2+1+i,:,:) ! (+,:,:)
          else
             data(n1/2+1+i,:,:) = data(n1/2+1+i,:,:)*( 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn1)) - 2.0/alpha + 1.0 )) ) ! (+,:,:)
          end if
       end do
       !! negative 1st axis
       nn1=int(n1/2)-1 ! number of negative elements on the 1st axis
       do i=0,nn1
          if( i .lt. int(0.5*alpha*float(nn1)) )then
             data(1+i,:,:) = data(1+i,:,:)*( 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn1)) - 1.0 )) ) ! (-,:,:)
          elseif( i.ge.int(0.5*alpha*float(nn1)) .and. i.le.(nn1-int(0.5*alpha*float(nn1))) )then
             data(1+i,:,:) = data(1+i,:,:) ! (-,:,:)
          else
             data(1+i,:,:) = data(1+i,:,:)*( 0.5 + 0.5*cos(pi*(2.0*float(i)/(alpha*float(nn1)) - 2.0/alpha + 1.0 )) ) ! (-,:,:)
          end if
       end do
       !!============================

       !!-------------------
       !! apply fan filter to (:,+,+)
       !!-------------------
          !! apply cosine fan filter on the 2nd axis
          nn2=n2-int(n2/2)-1 ! number of positive elements
          do j=0,nn2
             do k=0,int(float(j)*tan(phi))  ! 
                if( float(k) .lt. float(j)*tan(phi) )then
                   data(:,n2/2+1+j,n3/2+1+k) = data(:,n2/2+1+j,n3/2+1+k)*(0.5 + 0.5*cos(pi*(float(k)/(float(j)*tan(phi)) - 1.0 ))) ! (:,+,+)
                end if
             end do
          end do
          !! apply cosine fan filter on the 3rd axis
          nn3=n3-int(n3/2)-1 ! number of positive elements
          do k=0,nn3
             do j=0,int(float(k)*tan(phi))  ! 
                if( float(j) .lt. float(k)*tan(phi) )then
                   data(:,n2/2+1+j,n3/2+1+k) = data(:,n2/2+1+j,n3/2+1+k)*(0.5 + 0.5*cos(pi*(float(j)/(float(k)*tan(phi)) - 1.0 ))) ! (:,+,+)
                end if
             end do
          end do

       !!-------------------
       !! apply fan filter to (:,+,-)
       !!-------------------
          !! apply cosine fan filter on the 2nd axis
          nn2=n2-int(n2/2)-1 ! number of positive elements
          do j=0,nn2
             do k=0,int(float(j)*tan(phi))
                if( float(k) .lt. float(j)*tan(phi) )then
                   data(:,n2/2+1+j,n3/2-k) = data(:,n2/2+1+j,n3/2-k)*(0.5 + 0.5*cos(pi*(float(k)/(float(j)*tan(phi)) - 1.0 ))) ! (:,+,-)
                end if
             end do
          end do
          !! apply cosine fan filter on the 3rd axis
          nn3=int(n3/2)-1 ! number of negative elements
          do k=0,nn3
             do j=0,int(float(k)*tan(phi))  ! 
                if( float(j) .lt. float(k)*tan(phi) )then
                   data(:,n2/2+1+j,n3/2-k) = data(:,n2/2+1+j,n3/2-k)*(0.5 + 0.5*cos(pi*(float(j)/(float(k)*tan(phi)) - 1.0 ))) ! (:,+,-)
                end if
             end do
          end do

       !!-------------------
       !! apply fan filter to (:,-,-)
       !!-------------------
          !! apply cosine fan filter on the 2nd axis
          nn2=int(n2/2)-1 ! number of negative elements
          do j=0,nn2
             do k=0,int(float(j)*tan(phi))
                if( float(k) .lt. float(j)*tan(phi) )then
                   data(:,n2/2-j,n3/2-k) = data(:,n2/2-j,n3/2-k)*(0.5 + 0.5*cos(pi*(float(k)/(float(j)*tan(phi)) - 1.0 ))) ! (:,-,-)
                end if
             end do
          end do
          !! apply cosine fan filter on the 3rd axis
          nn3=int(n3/2)-1 ! number of negative elements
          do k=0,nn3
             do j=0,int(float(k)*tan(phi))  ! 
                if( float(j) .lt. float(k)*tan(phi) )then
                   data(:,n2/2-j,n3/2-k) = data(:,n2/2-j,n3/2-k)*(0.5 + 0.5*cos(pi*(float(j)/(float(k)*tan(phi)) - 1.0 ))) ! (:,-,-)
                end if
             end do
          end do

       !!-------------------
       !! apply fan filter to (:,-,+)
       !!-------------------
          !! apply cosine fan filter on the 2nd axis
          nn2=int(n2/2)-1 ! number of negative elements
          do j=0,nn2
             do k=0,int(float(j)*tan(phi))
                if( float(k) .lt. float(j)*tan(phi) )then
                   data(:,n2/2-j,n3/2+1+k) = data(:,n2/2-j,n3/2+1+k)*(0.5 + 0.5*cos(pi*(float(k)/(float(j)*tan(phi)) - 1.0 ))) ! (:,-,+)
                end if
             end do
          end do
          !! apply cosine fan filter on the 3rd axis
          nn3=n3-int(n3/2)-1 ! number of positive elements
          do k=0,nn3
             do j=0,int(float(k)*tan(phi))  ! 
                if( float(j) .lt. float(k)*tan(phi) )then
                   data(:,n2/2-j,n3/2+1+k) = data(:,n2/2-j,n3/2+1+k)*(0.5 + 0.5*cos(pi*(float(j)/(float(k)*tan(phi)) - 1.0 ))) ! (:,-,+)
                end if
             end do
          end do

       !!============================
       !! apply 2D cosine fan filter on the plane composed of the last two axes
       !! Now, keep data only in the chosen region
       if(flag.eq.1) then
!         # flag=1 keep the data in (+f,+kz,+kx) and (-f,-kz,-kx): downright-going wavefields
          ! flag=1: (+,+,+)axes and (-,-,-)axes will be stored, (+,-)axes and (-,+)axes will be removed.

          !! positive 1st axis
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = 0. ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = 0. ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = 0. ! (+,-,+)

          !! negative 1st axis
          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = 0. ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = 0. ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = data(1:n1/2,1:n2/2,1:n3/2) ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = 0. ! (-,-,+)

       elseif(flag.eq.-1)then
!         # flag=-1 keep the data in (+f,-kz,-kx) and (-f,+kz,+kx): upleft-going wavefields
          !! positive 1st axis
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = 0. ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = 0. ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = data(n1/2+1:n1,1:n2/2,1:n3/2) ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = 0. ! (+,-,+)

          !! negative 1st axis
          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = data(1:n1/2,n2/2+1:n2,n3/2+1:n3) ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = 0. ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = 0. ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = 0. ! (-,-,+)

       elseif(flag.eq.2)then
!         # flag=2 keep the data in (+f,-kz,+kx) and (-f,+kz,-kx): upright-going wavefields
          !! positive 1st axis
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = 0. ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = 0. ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = 0. ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = data(n1/2+1:n1,1:n2/2,n3/2+1:n3) ! (+,-,+)

          !! negative 1st axis
          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = 0. ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = data(1:n1/2,n2/2+1:n2,1:n3/2) ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = 0. ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = 0. ! (-,-,+)

       elseif(flag.eq.-2)then
!         # flag=-2 keep the data in (+f,+kz,-kx) and (-f,-kz,+kx): downleft-going wavefields
          !! positive 1st axis
          data(n1/2+1:n1,n2/2+1:n2,n3/2+1:n3) = 0. ! (+,+,+)
          data(n1/2+1:n1,n2/2+1:n2,1:n3/2) = data(n1/2+1:n1,n2/2+1:n2,1:n3/2) ! (+,+,-)
          data(n1/2+1:n1,1:n2/2,1:n3/2) = 0. ! (+,-,-)
          data(n1/2+1:n1,1:n2/2,n3/2+1:n3) = 0. ! (+,-,+)

          !! negative 1st axis
          data(1:n1/2,n2/2+1:n2,n3/2+1:n3) = 0. ! (-,+,+)
          data(1:n1/2,n2/2+1:n2,1:n3/2) = 0. ! (-,+,-)
          data(1:n1/2,1:n2/2,1:n3/2) = 0. ! (-,-,-)
          data(1:n1/2,1:n2/2,n3/2+1:n3) = data(1:n1/2,1:n2/2,n3/2+1:n3) ! (-,-,+)
       end if

    end if

  end subroutine filter3D


end module fkfilter3D_mod
